using System;
using System.IO;
using System.Threading;

namespace ViewState
{
	/// <summary>
	/// Tda FileVSDALC implementuje souborov loit pro ViewState. 
	/// Pro zajitn jedinenosti pstupu k ukldacmu adresi v rmci cel aplikace pouv koncept synchronizace vlken pomoc statickho mutexu.
	/// </summary>
	public class FilePersistor : BasePersistor
	{
		#region private members and constants
		private string mDirectory;
		private static Mutex storeMutex = new Mutex();
		#endregion

		#region constructors
		public FilePersistor(string directory, int objectExpirationTimeout) : base(objectExpirationTimeout)
		{			
			mDirectory = directory;			
		}
		#endregion

		#region public methods
		/// <summary>
		/// Abstraktn metoda pro natn ViewState z loit.
		/// </summary>
		/// <param name="viewStateID">Identifiktor natanho ViewState</param>
		/// <returns>Naten ViewState.</returns>
		public override object GetViewState(Guid viewStateID) 
		{	
			// pockame na umozneni vstupu do kriticke sekce
			storeMutex.WaitOne();
			try 
			{
				object deserializedViewStateBag = string.Empty;
				// zjistime jmeno souboru
				string fileName = GetFileName(viewStateID);
				// otevreni souboru
				if (File.Exists(fileName)) 
				{
					// oteven readeru
					StreamReader viewStateReader = new StreamReader(fileName);
					try 
					{
						// naten serializovanho ViewState
						string serializedViewState = viewStateReader.ReadToEnd();
						// deserializace ViewState
						deserializedViewStateBag = DeserializeViewState(serializedViewState);			
						// vrcen deserializovanho ViewState
						return deserializedViewStateBag;
					}
					finally 
					{
						// uzaven readeru
						viewStateReader.Close();
					}								
				}
				else 
				{
					throw new Exception("Specifikovan soubor s ViewState nenalezen.");
				}
			}
			finally 
			{
				// uvolnime kritickou sekci
				storeMutex.ReleaseMutex();
			}
		}

		/// <summary>
		/// Metoda pro ukldn ViewState do loit.
		/// </summary>
		/// <param name="viewStateID">Identifiktor ViewState</param>
		/// <param name="viewStateBag">Objekt obsahujc ukldan ViewState</param>
		public override void SaveViewState(Guid viewStateID, object viewStateBag) 
		{
			// pockame na uvolneni kriticke sekce
			storeMutex.WaitOne();
			try 
			{
				// zjitn jmna souboru pro ViewState
				string fileName = GetFileName(viewStateID);
				// otevren vstupnho streamu
				StreamWriter viewStateWriter = new StreamWriter(fileName, false);				
				try 
				{					
					// serializace ViewState do otevenho streamu
					SerializeViewState(viewStateBag, viewStateWriter);										
				}
				catch (Exception exc) 
				{
					throw new Exception("Chyba pi zpisu ViewState do souboru.", exc);
				}
				finally 
				{
					// uzaven vstupnho streamu
					viewStateWriter.Close();
				}
			}
			finally 
			{
				// uvolneni kriticke sekce
				storeMutex.ReleaseMutex();
			}
		}

		/// <summary>
		/// Abstraktn metoda pro vymazn vech uloench ViewState z loit.
		/// </summary>
		public override void DeleteApplicationViewStates() 
		{
			// pockame na uvolneni kriticke sekce
			storeMutex.WaitOne();
			try 
			{
				// zjitn vech soubor v adresi loit
				string[] viewStateFiles = Directory.GetFiles(mDirectory, "*");
				// v cyklu pes vechny soubory
				for (int fileIndex = viewStateFiles.GetLowerBound(0); fileIndex <= viewStateFiles.GetUpperBound(0); fileIndex++) 
				{
					try 
					{
						// vymazn souboru
						File.Delete(viewStateFiles[fileIndex]);
					}
					catch (Exception exc)
					{
						// tak ted fakt nevim co s tim... :-)
						// ale protoze je blbost odsud chybu hazet dal, tak ji vyhodim
						throw exc;
					}
				}
			}
			finally 
			{
				// uvolneni kriticke sekce
				storeMutex.ReleaseMutex();
			}
		}

		/// <summary>
		/// Metoda pro vymazn vech poloek ViewState s vyprenou platnost z loit.
		/// </summary>
		public override void DeleteExpiredViewStates() 
		{	
			// pockame na uvolneni kriticke sekce
			storeMutex.WaitOne();
			try 
			{
				// zjistime datum expirace 
				// DateTime expirationDateTime = ASD.Functions.ViewState.GetExpirationDateTime();
				DateTime expirationDateTime = DateTime.Now;
				// nateme vechny soubory v loiti
				string[] viewStateFiles = Directory.GetFiles(mDirectory, "*");
				// cyklus pes vechny soubory
				for (int fileIndex = viewStateFiles.GetLowerBound(0); fileIndex <= viewStateFiles.GetUpperBound(0); fileIndex++) 
				{
					try 
					{
						// zjistme jestli datum poslednho pstupu k souboru je star ne zjitn datum expirace
						if (File.GetLastAccessTime(viewStateFiles[fileIndex]) < expirationDateTime) 
						{
							// smazn soubory
							File.Delete(viewStateFiles[fileIndex]);
						}
					}
					catch (Exception exc) 
					{										
						throw exc;
					}
				}
			}
			finally 
			{
				// uvolnime kritickou sekci
				storeMutex.ReleaseMutex();
			}
		}
		#endregion

		#region private and protected methods
		protected virtual string GetFileName(Guid viewStateID) 
		{
			string fileName = mDirectory;
			if (!fileName.EndsWith("\\")) 
			{
				fileName += "\\";
			}
			fileName += "VIEW_STATE_" + viewStateID.ToString() + ".tmp";
			return fileName;
		}
		#endregion
	}
}

