using System;
using System.Timers;
using System.Threading;
using System.Web;

namespace ViewState
{
	/// <summary>
	/// Statick tda ViewState implementuje spolen statick metody pro prci s loiti ViewState.
	/// Tda pouv nvrhov vzor Singleton pro zajitn vytvoen pouze jedn instance objektu v rmci aplikace.
	/// </summary>
	public class StoreCleaner
	{
		#region private members and constants
		private static StoreCleaner mInstance;
		private int mTimeout;
		private System.Timers.Timer mTimer;						
		#endregion

		#region constructors and destructor
		private StoreCleaner(int timeout) 
		{
			// nastavime Timeout po casovac
			mTimeout = timeout;
			mTimer = new System.Timers.Timer((double) (timeout * 1000));
			// zajisteni opakovani timeoutu (nezastavi se po prvnim "tiku")
			mTimer.AutoReset = true;
			// registrace metody pro udalost casovace
			mTimer.Elapsed += new ElapsedEventHandler(ElapsedTimeEvent);		
			// spusteni casovace
			mTimer.Start();
		}		

		~StoreCleaner() 
		{
			try 
			{
				mTimer.Close();				
			}
			catch 
			{
			}
		}
		#endregion

		#region public members
		/// <summary>
		/// Statick metoda pro instanciaci a sputn cel thle mazac srandy.
		/// </summary>
		public static void StartAutomaticExpiredDeletion()
		{						
			if (mInstance == null)  
			{											
				int objectExpirationTime = GetSessionTimeout();
				if (objectExpirationTime == -1) 
				{
					// nepodarilo se zjistis session timeout, vezmeme hodnotu z konfiguracniho parametru
					objectExpirationTime = Configuration.DeleteExpiredPeriod;
				}
				else
				{					
					if (objectExpirationTime < Configuration.ObjectExpiration)
						objectExpirationTime = Configuration.DeleteExpiredPeriod;
				}
				// vytvorime instanci s definovanou casovou periodou
				mInstance = new StoreCleaner(objectExpirationTime);				
			}			
		}

		/// <summary>
		/// Metoda zastav asova pro automatick mazn.
		/// </summary>
		public static void StopAutomaticExpiredDeletion() 
		{
			if (mInstance != null) 
			{
				mInstance.StopInstanceTimer();
			}
		}
		
		/// <summary>
		/// Zjitn expiranho datumu vzhledem ke konfiguraci a aktulnmu datu.
		/// </summary>
		/// <returns></returns>
		public static DateTime GetExpirationDateTime() 
		{
			return DateTime.Now.AddSeconds(Configuration.ObjectExpiration * (-1));
		}
			
		/// <summary>
		/// Metoda implementuje vymazn vech uloench poloek ViewState v loiti. Voln tto metody je nejvhodnj (pro sprvnou funknost) provdt v global.asax.cs v udlostech Application_Start a Application_End.
		/// </summary>
		public static void DeleteApplicationViewStates() 
		{
			EViewStateStore configuredStore = Configuration.ViewStateStore;			
			int configuredExpiration = Configuration.ObjectExpiration;
			BasePersistor usedPersistor;
			switch (configuredStore) 
			{				
				case EViewStateStore.Database:
					string configuredConnectionString = Configuration.ConnectionString;
					usedPersistor = new DatabasePersistor(configuredConnectionString, configuredExpiration);
					break;
				case EViewStateStore.File:
					string configuredDirectory = Configuration.Directory;
					usedPersistor = new FilePersistor(configuredDirectory, configuredExpiration);
					break;
				case EViewStateStore.Cache:
					// v pripade cache se hodnoty mazou automaticky po vyprseni doby expirace
				default:
					// v defaultni implementaci take neni nutne se o mazani starat
					return;
			}			

			try 
			{
				// vymazani expirovanych polozek				
				usedPersistor.DeleteApplicationViewStates();
			}
			catch (Exception exc) 
			{
				throw exc;
			}
		}
		#endregion

		#region private methods
		private void StartInstanceTimer() 
		{			
			mTimer.Start();
		}
		
		private void StopInstanceTimer() 
		{
			mTimer.Stop();
		}				

		/// <summary>
		/// Metoda implementuje ovlada udlosti generovan asovaem. 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void ElapsedTimeEvent(object sender, ElapsedEventArgs e) 
		{
			// vymazani vyprsenych polozek ViewState z uloziste
			// zjisteni typu uloziste ViewState z konfigurace
			EViewStateStore configuredStore = Configuration.ViewStateStore;						
			int configuredExpiration = Configuration.ObjectExpiration;
			BasePersistor usedPersistor;
			// podle typu uloziste vytvorime instanci prislusne implementacni tridy			
			switch (configuredStore) 
			{								
				case EViewStateStore.Database:
					// uloziste implementovano v databazi
					string configuredConnectionString = Configuration.ConnectionString;
					usedPersistor = new DatabasePersistor(configuredConnectionString, configuredExpiration);
					break;
				case EViewStateStore.File:
					// uloziste implementovano v adresari na disku
					string configuredDirectory = Configuration.Directory;					
					usedPersistor = new FilePersistor(configuredDirectory, configuredExpiration);
					break;
				case EViewStateStore.Cache:
				default:					
					return;					
			}			
			// pokusime se smazat vsechny vyprsene stavy			
			try 
			{
				usedPersistor.DeleteExpiredViewStates();
			}
			catch (Exception exc) 
			{
				throw exc;
			}			
		}

		/// <summary>
		/// Metoda zjiuje nastaven Timeout aktulnho objektu Session.
		/// </summary>
		/// <returns>nastavena zivotnost Session, nebo -1 pokud se nepodari zjisti</returns>
		private static int GetSessionTimeout() 
		{
			try 
			{
				return HttpContext.Current.ApplicationInstance.Session.Timeout * 60;
			}
			catch (Exception exc) 
			{
				return -1;
			}
		}
		#endregion
	}
}
